@@ -181,6 +181,7 @@ void do_show_super_stats(int argc, char *argv[])
 	fprintf(out, "Superblock size = %d\n", sizeof(struct ext2_super_block));
 	fprintf(out, "Block size = %d, fragment size = %d\n",
 		EXT2_BLOCK_SIZE(fs->super), EXT2_FRAG_SIZE(fs->super));
+	fprintf(out, "Inode size = %d\n", EXT2_INODE_SIZE(fs->super));
 	fprintf(out, "%d inodes, %d free\n", fs->super->s_inodes_count,
 	        fs->super->s_free_inodes_count);
 	fprintf(out, "%d blocks, %d free, %d reserved, first block = %d\n",
@@ -189,7 +190,6 @@ void do_show_super_stats(int argc, char *argv[])
 	fprintf(out, "%d blocks per group\n", fs->super->s_blocks_per_group);
 	fprintf(out, "%d fragments per group\n", fs->super->s_frags_per_group);
 	fprintf(out, "%d inodes per group\n", EXT2_INODES_PER_GROUP(fs->super));
-	fprintf(out, "%d inodes per block\n", EXT2_INODES_PER_BLOCK(fs->super));
 	fprintf(out, "%ld group%s (%ld descriptors block%s)\n",
 		fs->group_desc_count, (fs->group_desc_count != 1) ? "s" : "",
 		fs->desc_blocks, (fs->desc_blocks != 1) ? "s" : "");
@@ -261,8 +261,14 @@ static void dump_inode(ino_t inode_num, struct ext2_inode inode)
 		inode.i_mode & 0777, inode.i_flags, inode.i_version);
 	fprintf(out, "User: %5d   Group: %5d   Size: %d\n",  
 		inode.i_uid, inode.i_gid, inode.i_size);
-	fprintf(out, "File ACL: %d    Directory ACL: %d\n",
-		inode.i_file_acl, inode.i_dir_acl);
+	if (fs->super->s_creator_os == EXT2_OS_HURD)
+		fprintf(out,
+			"File ACL: %d    Directory ACL: %d Translator: %d\n",
+			inode.i_file_acl, inode.i_dir_acl,
+			inode.osd1.hurd1.h_i_translator);
+	else
+		fprintf(out, "File ACL: %d    Directory ACL: %d\n",
+			inode.i_file_acl, inode.i_dir_acl);
 	fprintf(out, "Links: %d   Blockcount: %d\n", inode.i_links_count,
 		inode.i_blocks);
 #if HAVE_EXT2_FRAGS
@@ -625,6 +631,11 @@ void do_modify_inode(int argc, char *argv[])
 #endif
 	modify_u32(argv[0], "File acl", decimal_format, &inode.i_file_acl);
 	modify_u32(argv[0], "Directory acl", decimal_format, &inode.i_dir_acl);
+
+	if (fs->super->s_creator_os == EXT2_OS_HURD)
+		modify_u32(argv[0], "Translator Block",
+			    decimal_format, &inode.osd1.hurd1.h_i_translator);
+	
 	modify_u32(argv[0], "Fragment address", decimal_format, &inode.i_faddr);
 #if HAVE_EXT2_FRAGS
 	modify_u8(argv[0], "Fragment number", decimal_format, &inode.i_frag);
