@@ -91,6 +91,7 @@ static void show_stats(ext2_filsys fs)
 	int inodes, inodes_used, blocks, blocks_used;
 	int dir_links;
 	int num_files, num_links;
+	int frag_percent;
 
 	dir_links = 2 * fs_directory_count - 1;
 	num_files = fs_total_count - dir_links;
@@ -101,19 +102,22 @@ static void show_stats(ext2_filsys fs)
 	blocks = fs->super->s_blocks_count;
 	blocks_used = (fs->super->s_blocks_count -
 		       fs->super->s_free_blocks_count);
+
+	frag_percent = (10000 * fs_fragmented) / inodes_used;
+	frag_percent = (frag_percent + 5) / 10;
 	
 	if (!verbose) {
-		printf("%s: %d/%d files (%3.1f%% non-contiguous), %d/%d blocks\n",
+		printf("%s: %d/%d files (%0d.%d%% non-contiguous), %d/%d blocks\n",
 		       device_name, inodes_used, inodes,
-		       100.00 * fs_fragmented / inodes_used,
+		       frag_percent / 10, frag_percent % 10,
 		       blocks_used, blocks);
 		return;
 	}
 	printf ("\n%8d inode%s used (%d%%)\n", inodes_used,
 		(inodes_used != 1) ? "s" : "",
 		100 * inodes_used / inodes);
-	printf ("%8d non-contiguous inodes (%3.1f%%)\n",
-		fs_fragmented, 100.00 * fs_fragmented / inodes_used);
+	printf ("%8d non-contiguous inodes (%0d.%d%%)\n",
+		fs_fragmented, frag_percent / 10, frag_percent % 10);
 	printf ("         # of inodes with ind/dind/tind blocks: %d/%d/%d\n",
 		fs_ind_count, fs_dind_count, fs_tind_count);
 	printf ("%8d block%s used (%d%%)\n"
@@ -220,6 +224,24 @@ static void check_super_value(const char *descr, unsigned long value,
 	}
 }
 
+static void relocate_hint()
+{
+	static hint_issued = 0;
+
+	/* Only issue the hint once */
+	if (hint_issued)
+		return;
+
+	printf("Note: if there is several inode or block bitmap blocks\n"
+	       "which require relocation, or one part of the inode table\n"
+	       "which must be moved, you may wish to try running e2fsck\n"
+	       "the '-b 8193' option first.  The problem may lie only with\n"
+	       "the primary block group descriptor, and the backup block\n"
+	       "group descriptor may be OK.\n\n");
+	hint_issued = 1;
+}
+
+	
 static void check_super_block(ext2_filsys fs)
 {
 	blk_t	first_block, last_block;
@@ -285,6 +307,7 @@ static void check_super_block(ext2_filsys fs)
 		       "have been %u\n", s->s_blocks_per_group,
 		       should_be);
 		printf(corrupt_msg);
+		fatal_error(0);
 	}
 
 	should_be = (s->s_log_block_size == 0) ? 1 : 0;
@@ -293,6 +316,7 @@ static void check_super_block(ext2_filsys fs)
 		       "have been %u\n", s->s_first_data_block,
 		       should_be);
 		printf(corrupt_msg);
+		fatal_error(0);
 	}
 
 	/*
@@ -306,6 +330,7 @@ static void check_super_block(ext2_filsys fs)
 			last_block = fs->super->s_blocks_count;
 		if ((fs->group_desc[i].bg_block_bitmap < first_block) ||
 		    (fs->group_desc[i].bg_block_bitmap >= last_block)) {
+			relocate_hint();
 			printf("Block bitmap for group %d is not in group.  "
 			       "(block %u)\n",
 			       i, fs->group_desc[i].bg_block_bitmap);
@@ -319,6 +344,7 @@ static void check_super_block(ext2_filsys fs)
 		}
 		if ((fs->group_desc[i].bg_inode_bitmap < first_block) ||
 		    (fs->group_desc[i].bg_inode_bitmap >= last_block)) {
+			relocate_hint();
 			printf("Inode bitmap group %d not in group.  "
 			       "(block %u)\n",
 			       i, fs->group_desc[i].bg_inode_bitmap);
@@ -333,6 +359,7 @@ static void check_super_block(ext2_filsys fs)
 		if ((fs->group_desc[i].bg_inode_table < first_block) ||
 		    ((fs->group_desc[i].bg_inode_table +
 		      fs->inode_blocks_per_group - 1) >= last_block)) {
+			relocate_hint();
 			printf("Inode table for group %d not in group.  "
 			       "(block %u)\n",
 			       i, fs->group_desc[i].bg_inode_table);
@@ -601,7 +628,8 @@ restart:
 
 #ifdef	EXT2_CURRENT_REV
 	if (fs->super->s_rev_level > E2FSCK_CURRENT_REV) {
-		com_err(program_name, retval, "while trying to open %s",
+		com_err(program_name, EXT2_ET_REV_TOO_HIGH,
+			"while trying to open %s",
 			filesystem_name);
 		printf ("Get a newer version of e2fsck!\n");
 		fatal_error(0);
