@@ -14,9 +14,13 @@
  * 93/10/30	- Creation
  */
 
+#if HAVE_ERRNO_H
 #include <errno.h>
-#include <fcntl.h>
+#endif
+#if HAVE_UNISTD_H
 #include <unistd.h>
+#endif
+#include <fcntl.h>
 #include <sys/ioctl.h>
 
 #include <linux/ext2_fs.h>
@@ -25,13 +29,19 @@
 
 int fgetversion (const char * name, unsigned long * version)
 {
+#if HAVE_EXT2_IOCTLS
 	int fd;
 	int r;
 
-	fd = open (name, O_RDONLY);
+	fd = open (name, O_RDONLY|O_NONBLOCK);
 	if (fd == -1)
 		return - 1;
 	r = ioctl (fd, EXT2_IOC_GETVERSION, version);
 	close (fd);
 	return r;
+#else /* ! HAVE_EXT2_IOCTLS */
+	extern int errno;
+	errno = EOPNOTSUPP;
+	return -1;
+#endif /* ! HAVE_EXT2_IOCTLS */
 }
